#include "Fit/Fitter.h"
#include "Fit/BinData.h"
#include "Fit/Chi2FCN.h"
#include "TH1.h"
#include "TList.h"
#include "Math/WrappedMultiTF1.h"
#include "HFitInterface.h"
#include "TCanvas.h"
#include "TStyle.h"
 
 
// definition of shared parameter
// background function
int nParB = 5;
int iparB[5] = { 0,   
                 2, //                                 Shared (set to zero for now)  
                 6, // Xi(1820) top        amplitude
                 7, // Xi(1820) top+bottom mean        Shared
                 8  // Xi(1820) top+bottom sigma       Shared
};
 
// signal + background function
int nParSB = 8;
int iparSB[8] = { 1, 
                  2, //                                Shared (set to zero for now)
                  3, // Xi(1530) amplitude
                  4, // Xi(1530) mean
                  5, // Xi(1530) sigma
                  9, // Xi(1820)     bottom amplitude
                  7, // Xi(1820) top+bottom mean       Shared
                  8  // Xi(1820) top+bottom sigma      Shared
};

const int Npar = 10;

  double gAmplitude1 = 40;
  double gCenter1 = 1.53;
  double gSigma1 = 0.01;
  double gAmplitude2 = 40;
  double gAmplitude2bottom = 40;
  double gCenter2 = 1.82;
  double gSigma2 = 0.02;
  double par0[Npar] = { 5,5,0.0,
			gAmplitude1,gCenter1,gSigma1, //1530 bottom
			gAmplitude2,gCenter2,gSigma2,
			gAmplitude2bottom};//1820 top

 
// Create the GlobalCHi2 structure
 
struct GlobalChi2 {
  GlobalChi2(  ROOT::Math::IMultiGenFunction & f1,
	       ROOT::Math::IMultiGenFunction & f2) :
    fChi2_1(&f1), fChi2_2(&f2) {}
 
  double operator() (const double *par) const {
    double p1[nParB];
    for (int i = 0; i < nParB; ++i) p1[i] = par[iparB[i] ];
 
    double p2[nParSB];
    for (int i = 0; i < nParSB; ++i) p2[i] = par[iparSB[i] ];
 
    return (*fChi2_1)(p1) + (*fChi2_2)(p2);
  }
 
  const  ROOT::Math::IMultiGenFunction * fChi2_1;
  const  ROOT::Math::IMultiGenFunction * fChi2_2;
};

void combinedFit() {

  TFile *hFile1 = new TFile("allKmLam.root");
  TH1D *hB; hFile1->GetObject("CasStar_Kin_MMS_4",hB);

  TFile *hFile2 = new TFile("mAllXi.root");
  TH1D *hSB; hFile2->GetObject("hCasStar_MC_Kin_4",hSB);

  hB->Rebin(4);
  hSB->Rebin(4);
 
  TF1 * fB = new TF1("fB","pol1 + gaus(2)",0,100);

 
  // perform now global fit
  TF1 * fSB = new TF1("fSB","pol1 + gaus(2) + gaus(5)",0,100);//!!!!!
 
  ROOT::Math::WrappedMultiTF1 wfB(*fB,1);
  ROOT::Math::WrappedMultiTF1 wfSB(*fSB,1);
 
  ROOT::Fit::DataOptions opt;
  ROOT::Fit::DataRange rangeB;
  // set the data range
  rangeB.SetRange(1.7,1.9);//!!!!!
  ROOT::Fit::BinData dataB(opt,rangeB);
  ROOT::Fit::FillData(dataB, hB);
 
  ROOT::Fit::DataRange rangeSB;
  rangeSB.SetRange(1.5,1.9);//!!!!!
  ROOT::Fit::BinData dataSB(opt,rangeSB);
  ROOT::Fit::FillData(dataSB, hSB);
 
  ROOT::Fit::Chi2Function chi2_B(dataB, wfB);
  ROOT::Fit::Chi2Function chi2_SB(dataSB, wfSB);
 
  GlobalChi2 globalChi2(chi2_B, chi2_SB);
 
  ROOT::Fit::Fitter fitter;

 
  // create before the parameter settings in order to fix or set range on them
  fitter.Config().SetParamsSettings(Npar,par0);
  // fix 5-th parameter
  fitter.Config().ParSettings(2).Fix();
  fitter.Config().ParSettings(4).Fix();
  fitter.Config().ParSettings(5).Fix();
  fitter.Config().ParSettings(7).Fix();
  fitter.Config().ParSettings(8).Fix();


  // set limits on the third and 4-th parameter
  //fitter.Config().ParSettings(2).SetLimits(-10,-1.E-4);
  fitter.Config().ParSettings(3).SetLimits(0,10000);
  fitter.Config().ParSettings(3).SetStepSize(5);

  fitter.Config().ParSettings(9).SetLimits(0,10000);
  fitter.Config().ParSettings(9).SetStepSize(5);
 
  fitter.Config().MinimizerOptions().SetPrintLevel(0);
  fitter.Config().SetMinimizer("Minuit2","Migrad");
 
  // fit FCN function directly
  // (specify optionally data size and flag to indicate that is a chi2 fit)

  fitter.FitFCN(Npar,globalChi2,0,dataB.Size()+dataSB.Size(),true);
  ROOT::Fit::FitResult result = fitter.Result();
  result.Print(std::cout);
 
  TCanvas * c1 = new TCanvas("Simfit","Simultaneous fit of two histograms",
			     10,10,700,700);
  c1->Divide(1,2);
  c1->cd(1);
  gStyle->SetOptFit(1111);
 
  fB->SetFitResult( result, iparB);
  fB->SetRange(rangeB().first, rangeB().second);
  fB->SetLineColor(kBlue);
  hB->GetListOfFunctions()->Add(fB);
  hB->Draw();
 
  c1->cd(2);
  fSB->SetFitResult( result, iparSB);
  fSB->SetRange(rangeSB().first, rangeSB().second);
  fSB->SetLineColor(kRed);
  hSB->GetListOfFunctions()->Add(fSB);
  hSB->Draw();
 
 
}

